function [w,pos] = SiVM(V,K,distance)
% This function executes a Simplex Volume Maximization (SiVM).
% Input:  - V ... datamatrix [nxk] with n observations of dimension k
%         - K ... integer value [1x1] defining how many vertices are determined
%         - kins ... kind of metrik used, choose from: ['L2','L1','cosine','KL','Hellinger'] (string)
% Output: - w ... vertices of SiVM
%
% Authors: Julian Horst und Sebastian Riedel
% Date: 17.11.2012
pos = 0;

% selection of a random vertex
vj = V(1,:);
% distance calculation between all points V and the starting vertex vj
strecke_i = metrik(vj,V,distance);
% fixation of first SiVM-vertex
w(1,:) = V(strecke_i==max(strecke_i),:);
[pos(1),~] = find(strecke_i==max(strecke_i));

% distance calculation between all points V and the first SiVM-vertex w
strecke_j = metrik(w,V,distance);
% fixation of second SiVM-vertex
w(2,:) = V(strecke_j==max(strecke_j),:);
[pos(2),~] = find(strecke_j==max(strecke_j));


a = max(strecke_j); % scale factor
for k=2:K-1
    phi = 0;
    lambda = 0;
    rho = 0;
    % calculation of simplex volume
    for i = 1:k
        d = metrik(w(i,:),V,distance);
        phi = phi + a *d;
        lambda = lambda + (k-1)/2*d.^2;
        for j = i+1:k
            rho = rho + d.*metrik(w(j,:),V,distance);
        end
    end
    arg = phi+rho-lambda;
    % fixation of kth SiVM-vertex
    w(k+1,:)=V(arg == max(arg),:);
    [pos(k+1),~] = find(arg == max(arg));
end


    
