library(archetypes)
#Set the working directory where the file is placed.
m <- read.table('med1967.dat', sep = " ")
m <- m[1:20,]
sel <- c(48, 40 ,39, 33, 34, 36)
mpulg <- m[,sel] / (10 * 2.54)
smpulg <- scale(mpulg, center = sapply(mpulg,mean), scale = sapply(mpulg,sd)) 
Sx <- cov(smpulg)
D2 <- mahalanobis(smpulg, colMeans(smpulg), Sx)
qlchi <- which(D2 <= qchisq(0.95, df = dim(mpulg)[2])) 
lsmpulg <- smpulg[qlchi,]

data=lsmpulg
huge=200
n <- ncol(t(data))

#Vectors to save the computational time used by each iteration.
time_iter <- list()       
time_ini <- Sys.time()
comp_time <- c()

rss <- 1e+06
for(i in 1 : (n-2)){
  cat("New iteration:")
  print(i)
  for (g in (i+1) : (n-1)){
    for (h in (g+1) : n){ #NOTE: If we would like to compute four archetypes, we would have to add a fourth 
      #loop and so on and so forth.
      vect_arch_ini=c(i,g,h)
      
      k <- length(vect_arch_ini)
      
      x_gvv <- rbind(t(data), rep(huge, n))
      
      zs <- x_gvv[, vect_arch_ini] 
      zs <- as.matrix(zs)
      alphas <- matrix(0, nrow = k, ncol = n)
      for (j in 1 : n){
        alphas[, j] = coef(nnls(zs, x_gvv[,j]))
      }
      
      resid <- zs %*% alphas - x_gvv
      
      auxrss=max(svd(resid)$d) / n
      if(auxrss < rss){
        rss <- auxrss
        mejor=vect_arch_ini
      }
      
    }
  }
  
  time_iter[[i]] <- Sys.time()
  
  if(i == 1){
    comp_time[1] <- difftime(time_iter[[i]], time_ini, units = "mins")
    cat("Computational time in this iteration: \n")
    print(time_iter[[i]] - time_ini)
  }else{
    comp_time[i] <- difftime(time_iter[[i]], time_iter[[i - 1]], units = "mins")
    cat("Computational time in this iteration: \n")
    print(time_iter[[i]] - time_iter[[i - 1]])
  } 
  
  print(mejor)
  print(rss)
}
save(mejor,rss,file="best_200.RData")
cat("End of the programm")
