#This R code allows to obtain all the Figures 1, 2, 3 and 4 of the paper and their associated results: 
#Archetypoids: a new approach to define representative archetypal data
#And also the Figures 1 and 2 of the corresponding supplementary material

#Loading needed functions and packages:
source("stepArchetypesMod.R")
source("stepArchetypoids.R")
source("Functions_to_calculate_real_archetypes_with_swap2.R")

library(archetypes)
library(MASS)
library(tripack)
library(ellipse)
library(cluster)
library(data.table)
library(ggplot2)
library(apcluster)


###### Fig. 1 ######
load('nations.Rda')
s=nations
d=matrix(0,nrow=12,ncol=12)

for (i in 1:12){
for (j in 1:12){
d[i,j]=sqrt(s[i,i]-2*s[i,j]+s[j,j])
}
}
x=cmdscale(d)
 e=row.names(nations)
 norep=20
K=4
set.seed(2014)
lass10 <- stepArchetypesMod(data=x,k=K,verbose=FALSE,nrep=norep) 
ai <- bestModel(lass10[[1]]) #archetypes
 
an6 <-  stepArchetypoids(K,TRUE,x,lass10,FALSE,K-1) #nearest archetypoids

e[an6[[1]]]
e[an6[[3]]]
aw6 <-  stepArchetypoids(K,FALSE,x,lass10,FALSE,K-1) #which_alpha archetypoids
e[aw6[[1]]]
e[aw6[[3]]]

ini_arch=c()
 for (j in 1:4){
 ini_arch[j]=which.max(ai$betas[j,])
 }
 
temp <- realArchetypes(x,huge = 200, 4, ini_arch) #which beta archetypoid

#sivm
#write.table(x,file="x.txt",col.names=F,row.names=F)
#In Matlab
#http://www-kd.iai.uni-bonn.de/index.php?page=software_details&id=34
#load('x.txt')
#[w,po] = SiVM(x,4,'l2');
#po =
#     9    11     2     1
#End in Matlab
#In R
 e[c(9 ,   11   ,  2,     1)]
 
##Furthestsum
#In Matlab
#http://www.imm.dtu.dk/~mm/downloads/PCHA.zip
stream = RandStream('mt19937ar','Seed',0);
RandStream.setDefaultStream(stream);
X=x';
I=1:size(X,2);
noc=4;
i=FurthestSum(X(:,I),noc,ceil(length(I)*rand)); 
#i=   9     2     1    11 #the same that which_alpha and which_beta
#End in Matlab

plot(x[,1],x[,2],type='n',xlab='Dim 1',ylab='Dim 2',cex.axis=1.5,asp=1,xlim=c(-2,2),ylim=c(-2,2))
points(x[,1],x[,2],col=4,pch=16)
text(x[,1],x[,2],labels=e,cex=1.5,pos=3)
points(parameters(ai),pch=4,cex=2)

#solution in dimension k=11
x=cmdscale(d,k=11)
e=row.names(nations)
norep=20
K=4
set.seed(2014)
lass10 <- stepArchetypesMod(data=x,k=K,verbose=FALSE,nrep=norep) 
ai <- bestModel(lass10[[1]]) #archetypes
an6 <-  stepArchetypoids(K,TRUE,x,lass10,FALSE,K-1) #nearest archetypoids
aw6 <-  stepArchetypoids(K,FALSE,x,lass10,FALSE,K-1) #which_alpha archetypoids
ini_arch=c()
 for (j in 1:4){
 ini_arch[j]=which.max(ai$betas[j,])
 }
temp <- realArchetypes(x,huge = 200, 4, ini_arch) #which beta archetypoid
e[an6[[1]]]
e[aw6[[1]]]
e[temp$archet]

#Kernel-AA
#In Matlab
#http://www.imm.dtu.dk/~mm/downloads/PCHA.zip
#s=[9.00  4.83 5.28  3.44   4.72  4.50   3.83  3.50  2.39 3.06 5.39       3.17;
#   4.83  9.00 4.56  5.00   4.00  4.83   3.33  3.39  4.00 3.39 2.39       3.50;
#   5.28  4.56 9.00  5.17   4.11  4.00   3.61  2.94  5.50 5.44 3.17       5.11;
#   3.44  5.00 5.17  9.00   4.78  5.83   4.67  3.83  4.39 4.39 3.33       4.28;
#   4.72  4.00 4.11  4.78   9.00  3.44   4.00  4.22  3.67 5.06 5.94       4.72;
#   4.50  4.83 4.00  5.83   3.44  9.00   4.11  4.50  4.11 4.50 4.28       4.00;
#   3.83  3.33 3.61  4.67   4.00  4.11   9.00  4.83  3.00 4.17 5.94       4.44;
#   3.50  3.39 2.94  3.83   4.22  4.50   4.83  9.00  4.17 4.61 6.06       4.28;
#   2.39  4.00 5.50  4.39   3.67  4.11   3.00  4.17  9.00 5.72 2.56       5.06;
#   3.06  3.39 5.44  4.39   5.06  4.50   4.17  4.61  5.72 9.00 5.00       6.67;
#   5.39  2.39 3.17  3.33   5.94  4.28   5.94  6.06  2.56 5.00 9.00       3.56;
#   3.17  3.50 5.11  4.28   4.72  4.00   4.44  4.28  5.06 6.67 3.56       9.00];
#
#noc=4;
#I=1:size(s,2);
#U=1:size(s,2);
#[S,C,SSE,varexpl]=PCHAkernel(s,noc,I,U);
###### Fig. 1 end ######


###### Fig. 2 ######
#Fig 2 (a)
#Data:
x=c(1,-1,-1,1,0,-.5,.5)
y=c(1,1,-1,-1,.5,0,0) 
X=data.frame(cbind(x,y))
#plot(X)
plot(X,type='n',asp=1)
text(X[,1],X[,2],1:7)

#2 archetypes without standardizing:
set.seed(2013)
lass10 <- stepArchetypesMod(data=X,k=2,verbose=FALSE,nrep=20) #computed omly for k=2
ai <- bestModel(lass10[[1]]) #archetypes
# ai$betas
 
 
#2 archetypoids without standardizing, combinatorial solution:
huge=200
data=X

k <- 2
n <- ncol(t(data))

rss <- 1e+06
 
for (g in 1:(n-1)){
 for (h in (g+1):n){
  vect_arch_ini=c(g,h)
  k <- length(vect_arch_ini)
  n <- ncol(t(data))

  x_gvv <- rbind(t(data), rep(huge, n))
  zs <- x_gvv[, vect_arch_ini]  
  zs <- as.matrix(zs)
  alphas <- matrix(0, nrow = k, ncol = n)
  
  for (j in 1 : n){
   alphas[, j] = coef(nnls(zs, x_gvv[,j]))
  }
    
  resid <- zs %*% alphas - x_gvv 
  auxrss=max(svd(resid)$d) / n
  cat("possible archetypoids", vect_arch_ini, "rss" , auxrss, "\n")
  
  if(auxrss < rss){
   rss <- auxrss
   mejor=vect_arch_ini
  }
 }
}

#pdf("Fig1_a.pdf",useDingbats=FALSE)
plot(X,type='n',asp=1,xlab='',ylab='',cex.axis=1.5)
text(X[,1],X[,2],1:7,cex=2)
points(X[6:7,1],X[6:7,2],pch=1,cex=5)
points(parameters(ai),pch=4,cex=2)
#dev.off()


#Fig 2 (b)
#Data:
a=sqrt(0.75)
x=c(0.4,-.4,1,-1,0.5,-.5,0.5,-.5)
y=c(0.1,-0.1,0,0,a,-a,-a,a) 

x=c(1,0.5,-0.5,-1,-.5,0.5,0.4,-.4)
y=c(0,a,a,0,-a,-a,0.1,-0.1) 

X=data.frame(cbind(x,y))
#plot(X)
plot(X,type='n',asp=1)
text(X[,1],X[,2],1:8)

#2 archetypes without standardizing:
set.seed(2013)
lass10 <- stepArchetypesMod(data=X,k=2,verbose=FALSE,nrep=20) #computed omly for k=2
ai <- bestModel(lass10[[1]]) #archetypes
 
#2 archetypoids without standardizing, combinatorial solution:
huge=200
data=X

k <- 2
n <- ncol(t(data))

rss <- 1e+06
 
for (g in 1:(n-1)){
 for (h in (g+1):n){
  vect_arch_ini=c(g,h)
  k <- length(vect_arch_ini)
  n <- ncol(t(data))

  x_gvv <- rbind(t(data), rep(huge, n))
  zs <- x_gvv[, vect_arch_ini]  
  zs <- as.matrix(zs)
  alphas <- matrix(0, nrow = k, ncol = n)
  
  for (j in 1 : n){
   alphas[, j] = coef(nnls(zs, x_gvv[,j]))
  }
    
  resid <- zs %*% alphas - x_gvv 
  auxrss=max(svd(resid)$d) / n
  cat("possible archetypoids", vect_arch_ini, "rss" , auxrss, "\n")
   
  if(auxrss < rss){
   rss <- auxrss
   mejor=vect_arch_ini
  }
 }
}

#mejor 
#[1] 7 8 #this is the best
#pdf("Fig1_b.pdf",useDingbats=FALSE) 
plot(X,type='n',asp=1,xlab='',ylab='',cex.axis=1.5,ylim=c(-1,1))
text(X[,1],X[,2],1:8,cex=2)
points(X[7:8,1],X[7:8,2],pch=1,cex=5)
points(parameters(ai),pch=4,cex=2)
#dev.off()

#Nearest individuals to archetypes:
ras=rbind( parameters(ai),X)
dras=dist(ras,method="euclidean",diag=F,upper=T,p=2)

i=2
mdras=as.matrix(dras)
diag(mdras)=100000000000

which.min(mdras[1,]) - (i)
min(mdras[1,])
#6 
#4 --> four
which.min(mdras[2,]) - (i)
min(mdras[2,])
#3 
#1 --> one

###### Fig. 2 end ######



###### Fig. 3 ######
#Fig 3 (a)
set.seed(2013)
huge=200
m=50
mu=c(1,1)
Sigma=matrix(c(1,0.8,0.8,1),nrow=2)
Y=mvrnorm(m , mu, Sigma) #We have noted that either depending on the platform (windows,linux) 
#or depending on the R version, the Y matrix may be generated with exchanged columns: column1 
#corresponding to column2 and otherwise.
#This figure is generated with Windows. In case of, data can be loaded from Y <- read.table("Y.txt")
X=Y
plot(X)
 
#k=2
set.seed(2013)
lass10 <- stepArchetypesMod(data=X,k=2,verbose=FALSE,nrep=20) 
ai <- bestModel(lass10[[1]]) #archetypes
 
an3 <- stepArchetypoids(2,TRUE,X,lass10,FALSE,1) #nearest archetypoids

#pdf("Fig2_a.pdf",useDingbats=FALSE)   
plot(X,xlab='',ylab='',cex.axis=1.5,xaxt="n",yaxt="n",xlim=c(-2,4),ylim=c(-2,4)) 
points(X[an3[[1]],1],X[an3[[1]],2],pch=16,cex=2)
points(parameters(ai),pch=4,cex=2)
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
#dev.off()


#Fig 3 (b)
#k=4
lass10 <- stepArchetypesMod(data=X,k=4,verbose=FALSE,nrep=20) 
ai <- bestModel(lass10[[1]]) #archetypes
 
an3 <-  stepArchetypoids(4,TRUE,X,lass10,FALSE,3) #nearest archetypoids

#pdf("Fig2_b.pdf",useDingbats=FALSE)   
plot(X,xlab='',ylab='',cex.axis=1.5,xaxt="n",yaxt="n",xlim=c(-2,4),ylim=c(-2,4)) 
points(X[an3[[1]],1],X[an3[[1]],2],pch=16,cex=2)
points(parameters(ai),pch=4,cex=2)
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
#dev.off() 


#Fig 3 (c)
#k=8
#Using ginv for obtaining the inverse

ginv.zalphasfn <- function(alphas, x, ...) {
 require(MASS)  
 return(t(ginv(alphas %*% t(alphas)) %*% alphas %*% t(x)))
}

stepArchetypes_mod <- function (data, k, nrep = 3, verbose = TRUE){
 mycall <- match.call()
 as <- list()
 for (i in 1:length(k)) {
  as[[i]] <- list()
  class(as[[i]]) <- "repArchetypes"
  for (j in seq_len(nrep)) {
   if (verbose) 
    cat("\n*** k=", k[i], ", rep=", j, ":\n", sep = "")
    as[[i]][[j]] <- archetypes(data, k = k[i], 
                               family = archetypesFamily("original",
                                                         scalefn = no.scalefn,rescalefn = no.rescalefn,
                                                         zalphasfn=ginv.zalphasfn))
   }
  }
  return(structure(as, class = "stepArchetypes", call = mycall))
}

set.seed(2013)
lass10 <- stepArchetypes_mod(data=X,k=8,verbose=FALSE,nrep=500) 
ai <- bestModel(lass10[[1]]) #archetypes
 
an3 <- stepArchetypoids(8,TRUE,X,lass10,FALSE,7) #nearest archetypoids
 
#pdf("Fig2_c.pdf",useDingbats=FALSE)   
plot(X,xlab='',ylab='',cex.axis=1.5,xaxt="n",yaxt="n",xlim=c(-2,4),ylim=c(-2,4)) 
points(X[an3[[1]],1],X[an3[[1]],2],pch=16,cex=2)
points(parameters(ai),pch=4,cex=2)
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
#dev.off() 


###### Fig. 3 end ######



###### Fig. 1 supplementary material ######
#Fig 1 (a)
Y <- read.table("Y.txt")
X=Y

huge=200
m=50
mu=c(1,1)
Sigma=matrix(c(1,0.8,0.8,1),nrow=2)

set.seed(2013)
#The following loop lasts some minuts. 
#If preferred, you can load the results in As.RData by means of load("As.RData")
A=matrix(0,nrow=400,ncol=2)  #archetypes
AN=matrix(0,nrow=400,ncol=2) #nearest archetypoids
AW=matrix(0,nrow=400,ncol=2) #which archetypoids

for (g in 1:100){
 Y=mvrnorm(m , mu, Sigma)
 D2 <- mahalanobis(Y, mu, Sigma)
 qlchi=which(D2<=qchisq(0.95,df=2))
 X=Y[qlchi,]
  
 lass10 <- stepArchetypes_mod(data = X, k = 4, verbose = FALSE, nrep = 20)  
 ai <- bestModel(lass10[[1]])
 A[((g-1)*4+1):(g*4),]=parameters(ai)
 
 res4_mahal <- stepArchetypoids(4,TRUE,X,ai,lass10,FALSE,3)
 AN[((g-1)*4+1):(g*4),]=X[res4_mahal[[1]],]
 
 res4_mahal_which <- stepArchetypoids(4,FALSE,X,ai,lass10,FALSE,3)
 AW[((g-1)*4+1):(g*4),]=X[res4_mahal_which[[1]],]
 cat(g)
}
#save(A, AN, AW, file = "As.RData") 
#load("As.RData")

#windows()
#pdf("Fig3_a.pdf",useDingbats=FALSE) 
plot(X,type='n',asp=1,xlab='',ylab='',cex.axis=1.5,xlim=c(-2,4),ylim=c(-2,4),xaxt="n",yaxt="n") 
points(AN,pch=16)
points(A,pch=4)
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5,mgp=c(1,2,0),tcl=-1)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5,mgp=c(1,2,0),tcl=-1)
#dev.off()


#Fig 1 (b)
#Y <- read.table("Y.txt")
#X=Y
#load("As.RData")

mu=c(1,1)
Sigma=matrix(c(1,0.8,0.8,1),nrow=2)

#image, see if a grid point is inside the convex hull.
gridx=seq(-2,4,0.05)
gridy=seq(-2,4,0.05)
n=length(gridx)
m=length(gridy)
z=matrix(0,nrow=length(gridx),ncol=length(gridy))

#windows()
#pdf("Fig3_b.pdf",useDingbats=FALSE) 
plot(X,type='n',asp=1,xlab='',ylab='',cex.axis=1.5,xlim=c(-2,4),ylim=c(-2,4),xaxt="n",yaxt="n") 
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5,mgp=c(1,2,0),tcl=-1)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5,mgp=c(1,2,0),tcl=-1)

for (g in 1:100){
 SA=AN[((g-1)*4+1):(g*4),] #A for archetypes.
 tr <- tri.mesh(SA[,1],SA[,2])
 for (i in 1:n){
  for (j in 1:m){
   z[i,j]=z[i,j]+ in.convex.hull(tr,gridx[i],gridy[j]);
  }
 }
}
image(gridx,gridy,z,col=gray((0:64)/64),xlab='',ylab='') #archetypoids
lines(ellipse(Sigma,centre=mu),col=3,lty=2)
lines(ellipse(Sigma,centre=mu,level=.5),col=4,lty=4)
#dev.off()


#Fig 1 (c)
m=50
mu=c(1,1)
Sigma=matrix(c(1,0.8,0.8,1),nrow=2)

medoides=matrix(0,nrow=400,ncol=2) #medoids
centros=matrix(0,nrow=400,ncol=2)  #centers, k-means

set.seed(2013)
for (g in 1:100){
 Y=mvrnorm(m , mu, Sigma)
 #Linux:
 #Y1=mvrnorm(m , mu, Sigma)
 #Y=Y1
 #Y[,1]=Y1[,2]
 #Y[,2]=Y1[,1]
 D2 <- mahalanobis(Y, mu, Sigma)
 qlchi=which(D2<=qchisq(0.95,df=2))
 X=Y[qlchi,]

 #obtaining the 4 medoids and kmeans of X, save them and then represent them.
 pamx=pam(X,4)

 medoides[((g-1)*4+1):(g*4),]=pamx$medoids
 mx=kmeans(X,4)
 centros[((g-1)*4+1):(g*4),]=mx$centers
}

#windows()
#pdf("Fig3_c.pdf",useDingbats=FALSE) 
plot(X,type='n',asp=1,xlab='',ylab='',cex.axis=1.5,xlim=c(-2,4),ylim=c(-2,4),xaxt="n",yaxt="n") 
points(AN,pch=16)
points(A,pch=4)
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5,mgp=c(1,2,0),tcl=-1)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5,mgp=c(1,2,0),tcl=-1)
points(medoides,pch=16,col=2)
points(centros,pch=4,col=2)
#dev.off()

###### Fig. 1 supplementary material end######



###### Stability  (Fig. 2 supplementary material)######
#Leave-one-out archetypoids
set.seed(2013)

huge=200
m=50
mu=c(1,1)
Sigma=matrix(c(1,0.8,0.8,1),nrow=2)
Y=mvrnorm(m,mu,Sigma)
#Y <- read.table("Y.txt")

#The following loop lasts some minuts. 
#If preferred, you can load the results in AsLOU.RData by means of load("AsLOU.RData")
A=matrix(0,nrow=200,ncol=2)  #archetypes
AN=matrix(0,nrow=200,ncol=2) #nearest archetypoids
AW=matrix(0,nrow=200,ncol=2) #which archetypoids 

for (g in 1:50){
 X=Y[-g,]
 
 lass10 <- stepArchetypes_mod(data = X, k = 4, verbose = FALSE, nrep = 20) 
 ai <- bestModel(lass10[[1]])
 
 A[((g-1)*4+1):(g*4),]=parameters(ai)
  
 res4_mahal <- stepArchetypoids(4,TRUE,X,lass10,FALSE,3)
 AN[((g-1)*4+1):(g*4),]=X[res4_mahal[[1]],]
 
 res4_mahal_which <- stepArchetypoids(4,FALSE,X,lass10,FALSE,3)
 AW[((g-1)*4+1):(g*4),]=X[res4_mahal_which[[1]],]

 cat(g)
}

#save(A, AN, AW, file = "AsLOU.RData")
#load("AsLOU.RData")

pats <- do.call(paste,c(as.data.frame(AN),sep="\r"))
pats <- factor(pats,levels=unique(pats))
#cbind(unique(AN),Freq=as.vector(table(pats)))
gvv <- cbind(unique(AN),Freq=as.vector(table(pats))) #nearest archetypoids

colnames(gvv) <- c("x","y","Count")
DT <- data.table(gvv,key='x,y')
count <- DT$Count
#pdf("Fig4_a.pdf",useDingbats=FALSE) 
qplot(x,y,data=DT[,sum(Count),by='x,y'],size=count, 
      xlim=c(-2,4),ylim=c(-2,4))+theme(text=element_text(size=27),
                                       axis.text.x=element_text(angle=90,vjust=1)) 
#dev.off()


##Leave-one-out medoids
set.seed(2013)
m=50
mu=c(1,1)
Sigma=matrix(c(1,0.8,0.8,1),nrow=2)
Y=mvrnorm(m , mu, Sigma)

#The following loop lasts some minuts. 
#If preferred, you can load the results in MLOU.RData by means of load("MLOU.RData")
A=matrix(0,nrow=200,ncol=2)  #means
AN=matrix(0,nrow=200,ncol=2) #medoids

for (g in 1:50){
 X=Y[-g,]
 pamx=pam(X,4)
 pa1=pamx$medoids
 mx=kmeans(X,4)
 pa2=mx$centers
  
 A[((g-1)*4+1):(g*4),]=pa2
 AN[((g-1)*4+1):(g*4),]=pa1
 cat(g)
}

#save(A, AN, AW, file = "MLOU.RData")
#load("MLOU.RData")

pats_AN <- do.call(paste,c(as.data.frame(AN),sep="\r"))
pats_AN <- factor(pats_AN,levels=unique(pats_AN))
gvv_AN <- cbind(unique(AN),Freq=as.vector(table(pats_AN))) #k-medoids

colnames(gvv_AN) <- c("x","y","Count")
DT <- data.table(gvv_AN,key='x,y')
count <- DT$Count
#pdf("Fig4_b.pdf",useDingbats=FALSE)
qplot(x, y, data=DT[,sum(Count),by='x,y'],size=count,
      xlim=c(-1,3),ylim=c(-1,3))+theme(text=element_text(size=27),
                                       axis.text.x = element_text(angle=90,vjust=1)) 
#dev.off()

###### Stability end ######



###### Fig. 4 ######
#Same data as Fig. 3b
Y <- read.table("Y.txt")
X=Y

set.seed(2013)
lass10 <- stepArchetypesMod(data=X,k=4,verbose=FALSE,nrep=20) 
ai <- bestModel(lass10[[1]]) #archetypes

an3 <-  stepArchetypoids(4,TRUE,X,lass10,FALSE,3) #nearest archetypoids


#Fig 4 (a)
#SiVM
#In Matlab:
#http://www-kd.iai.uni-bonn.de/index.php?page=software_details&id=34
#load 'Y.txt'
#[w,po] = SiVM(Y,4,'l2');
# w gives
#   -1.9458   -1.0017
#    3.6564    2.5933
#    0.4878    1.3117
#    0.1782   -0.8794
# po gives  6    46    24    10
plot(X,xlab='',ylab='',cex.axis=1.5,xlim=c(-2,4),ylim=c(-2,4),xaxt="n",yaxt="n") 
points(X[an3[[1]],1],X[an3[[1]],2],pch=16,cex=2)
points(parameters(ai),pch=4,cex=2)
pa=c(6,46,24,10) #results from SiVM from matlab
points(X[pa,1],X[pa,2],pch=0,cex=2) #square
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)

#End in Matlab.

#Fig 4 (b)
#In Matlab:
#http://www.cis.jhu.edu/~ehsan/Codes/SMRS_v1.0.zip

#load 'Y.txt'
#% find the representatives via sparse modelling
#[repInd,C] = smrs(Y',5,0); %alpha 5 en run-smrs
#repInd gives 46 17
                  
#sInd = findRep(C,0.9999,2)
#sInd gives 46 17 9 6
#End in Matlab.

#pdf("Fig5_a.pdf",useDingbats=FALSE) 
plot(X,xlab='',ylab='',cex.axis=1.5,xlim=c(-2,4),ylim=c(-2,4),xaxt="n",yaxt="n") 
points(X[an3[[1]],1],X[an3[[1]],2],pch=16,cex=2)
points(parameters(ai),pch=4,cex=2)
pa=c(46,17,9,6) #results from smr from matlab
points(X[pa,1],X[pa,2],pch=0,cex=2) #square
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
#dev.off()


#Fig 4 (c)
#Obtaining the four ones: 
s1 <- negDistMat(X, r=2)
ap <- apcluster(s1,q=.3)
#ap
#APResult object
#Number of samples     =  50 
#Number of iterations  =  141 
#Input preference      =  -5.007583 
#Sum of similarities   =  -21.84837 
#Sum of preferences    =  -20.03033 
#Net similarity        =  -41.8787 
#Number of clusters    =  4 
#Exemplars: 2 5 16 41
#Clusters:
#Cluster 1, exemplar 2:
#2 7 15 26 29 30 34 39 43 45 48 50
#Cluster 2, exemplar 5:
#4 5 9 46
#Cluster 3, exemplar 16:
#1 3 8 11 13 16 21 22 23 24 25 27 28 31 33 35 36 37 42 44 47 49
#Cluster 4, exemplar 41:
#6 10 12 14 17 18 19 20 32 38 40 41

#pdf("Fig5_b.pdf",useDingbats=FALSE) 
plot(X,xlab='',ylab='',cex.axis=1.5,xlim=c(-2,4),ylim=c(-2,4),xaxt="n",yaxt="n") 
points(X[an3[[1]],1],X[an3[[1]],2],pch=16,cex=2)
points(parameters(ai),pch=4,cex=2)
pa1=c(2,5,16,41)
points(X[pa1,1],X[pa1,2],pch=0,cex=2) #square
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
#dev.off()


#Fig 4 (d)
#In matlab:
#https://sites.google.com/site/nicolasgillis/code
#load 'Y.txt';
#M=Y';
#r=4;
#epsilon=0.000001;

#k1 gives the representatives
#[k1, X] = hottopixx_cplex(M, epsilon, r, [], 1, false);
#k1 gives 3, 35, 14, 20.
#End in Matlab

#pdf("Fig5_c.pdf",useDingbats=FALSE) 
plot(X,xlab='',ylab='',cex.axis=1.5,xlim=c(-2,4),ylim=c(-2,4),xaxt="n",yaxt="n")
points(X[an3[[1]],1],X[an3[[1]],2],pch=16,cex=2)
points(parameters(ai),pch=4,cex=2)
pa=c(3,35,14,20) #results from Hottopix from matlab
points(X[pa,1],X[pa,2],pch=0,cex=2) #square
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
#dev.off()


#Fig 4 (e)
#In Matlab:
#http://www.shakirm.com/code/EFLVM/index.html
#load 'Y.txt'
#case 'bpm'
#config = struct(...
# 'nSamples', 4000, ...
# 'burnin', 2000, ...
# 'nLeaps',  80, ...
# 'stepSize', 5e-3, ...
# 'thin', 1, ...
# 'saveout', 1);
#               
#dataName = 'bpmmodel';
#model = 'bpm';
#K = 4;
#seed = 1;
#       
#data=Y;
#%exptEFLVM(dataName,model,K, seed);     
#      
#%Get config settings
#options = getConfigEFLVM(dataName, model);
#burnin = options.burnin;
#thin = options.thin;
#saveOut = options.saveout;
#
#setSeed(seed);
#       
#inferFun = @inferBPM_hmc;
#predFun = @predProbBPM_mcmc;
#
#%run HMC
#data.X=Y;
#postDist = inferFun(data, options, K);
#
#[N,D] = size(data.X);
#
#[S,LL] = size(postDist.samples);
#omega = postDist.samples(end,:);
#[a, rho, pi, theta] = extractParams_bpm(omega, K, N, D);
#
#max(pi)
#
#0.7929    0.9358    0.9924    0.7904
#pi
#The highest membership for each group is respectively: 6,10,40,39
#End in Matlab

#pdf("Fig5_d.pdf",useDingbats=FALSE) 
plot(X,xlab='',ylab='',cex.axis=1.5,xlim=c(-2,4),ylim=c(-2,4),xaxt="n",yaxt="n") 
points(X[an3[[1]],1],X[an3[[1]],2],pch=16,cex=2)
points(parameters(ai),pch=4,cex=2)
pa=c(6,10,40,39) #results from bpm from matlab
points(X[pa,1],X[pa,2],pch=0,cex=2) #square
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
#dev.off()


#Fig. 4f
library(e1071)
#Obtaining the 4 medoids and kmeans of X, save them and then represent them:
pamx=pam(X,4)
pa1=pamx$medoids

mx=kmeans(X,4)
pa2=mx$centers

fu=cmeans(X,4)
pa3=fu$centers

#pdf("Fig5_e.pdf",useDingbats=FALSE) 
plot(X,xlab='',ylab='',cex.axis=1.5,xlim=c(-2,4),ylim=c(-2,4),xaxt="n",yaxt="n")
points(X[an3[[1]],1],X[an3[[1]],2],pch=16,cex=2)
points(parameters(ai),pch=4,cex=2)
points(pa1,pch=0,cex=2) #square
points(pa2,pch=2,cex=2) #triangle
points(pa3,pch=5,cex=2) #diamond
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
#dev.off()

#The fast and robust recursive algorithm for separable NMF
#In matlab:
#https://sites.google.com/site/nicolasgillis/code
# load 'Y.txt'
# [J,normM,U] = FastSepNMF(Y',4,0);
# J
###### Fig. 4 end ######



