#First experiment of the supplementary material of the paper: 
#Archetypoids: a new approach to define representative archetypal data


#Loading needed functions and packages:
source("stepArchetypesMod.R")
source("stepArchetypoids.R")
source("Functions_to_calculate_real_archetypes_with_swap2.R")

library(archetypes)

#Bivariate Normal data
Y=read.table('Y.txt')
set.seed(2014)
lass10 <- stepArchetypesMod(data=Y,k=4,verbose=FALSE,nrep=20) 
ai <- bestModel(lass10[[1]]) #archetypes
 
an3 <-  stepArchetypoids(4,FALSE,Y,lass10,FALSE,3) #which_alfa archetypoids

plot(Y,xlab='',ylab='',cex.axis=1.5,xaxt="n",yaxt="n",xlim=c(-2,4),ylim=c(-2,4)) 
axis(1,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
axis(2,at=seq(-2,4,1),labels=seq(-2,4,1),cex.axis=1.5)
 text(Y[an3[[1]],1],Y[an3[[1]],2],la=an3[[1]],cex=1.5)

K=4
x=cmdscale(as.matrix(dist(Y)),k=2,eig=T)
x=x$points
norep=20
set.seed(2014)
lass10 <- stepArchetypesMod(data=x,k=K,verbose=FALSE,nrep=norep) 
ai <- bestModel(lass10[[1]]) #archetypes
 
an6 <-  stepArchetypoids(K,FALSE,x,lass10,FALSE,K-1) #which_alpha archetypoids

plot(x,xlab='',ylab='',cex.axis=1.5,xaxt="n",yaxt="n",xlim=c(-4,4),ylim=c(-1,1)) 
axis(1,at=seq(-4,4,1),labels=seq(-4,4,1),cex.axis=1.5)
axis(2,at=seq(-1,1,.5),labels=seq(-1,1,.5),cex.axis=1.5)
 text(x[an6[[1]],1],x[an6[[1]],2],la=an3[[1]],cex=1.5)
 
#Uniform

m=50
set.seed(214)
r1=runif(m,min=-1,max=1)
r2=runif(m,min=-1,max=1)

X=cbind(r1,r2)

K=4

set.seed(2014)
lass10 <- stepArchetypesMod(data=X,k=K,verbose=FALSE,nrep=norep) 
ai <- bestModel(lass10[[1]]) #archetypes
 
an3 <-  stepArchetypoids(K,TRUE,X,lass10,FALSE,K-1) #nearest archetypoids
aw3 <-  stepArchetypoids(K,FALSE,X,lass10,FALSE,K-1) #which_alpha archetypoids

ini_arch=c()
for (j in 1:K){
ini_arch[j]=which.max(ai$betas[j,])
}

temp <- realArchetypes(X,huge = 200, K, ini_arch) #which_beta archetypoids, equals to which_alpha archetypoids



plot(X,xlab='',ylab='',cex.axis=1.5,xaxt="n",yaxt="n",xlim=c(-1,1),ylim=c(-1,1)) 
axis(1,at=seq(-1,1,.5),labels=seq(-1,1,.5),cex.axis=1.5)
axis(2,at=seq(-1,1,.5),labels=seq(-1,1,.5),cex.axis=1.5)
 text(X[temp$archet,1],X[temp$archet,2],la=temp$archet,cex=1.5)


K=4
 x=cmdscale(as.matrix(dist(X)),k=2,eig=T)
x=x$points
norep=20
set.seed(2014)
lass10 <- stepArchetypesMod(data=x,k=K,verbose=FALSE,nrep=norep) 
ai <- bestModel(lass10[[1]]) #archetypes
 
an6 <-  stepArchetypoids(K,TRUE,x,lass10,FALSE,K-1) #nearest archetypoids

aw6 <-  stepArchetypoids(K,FALSE,x,lass10,FALSE,K-1) #which_alpha archetypoids

ini_arch=c()
for (j in 1:K){
ini_arch[j]=which.max(ai$betas[j,])
}

temp <- realArchetypes(x,huge = 200, K, ini_arch)#which_beta archetypoids, equals to which_alpha archetypoids


plot(x,xlab='',ylab='',cex.axis=1.5,xaxt="n",yaxt="n",xlim=c(-1.5,1),ylim=c(-1.5,1)) 
axis(1,at=seq(-1.5,1,.5),labels=seq(-1.5,1,.5),cex.axis=1.5)
axis(2,at=seq(-1.5,1,.5),labels=seq(-1.5,1,.5),cex.axis=1.5)
 text(x[temp$archet,1],x[temp$archet,2],la=temp$archet,cex=1.5)

#combinatorial solution

data=X
#data=x 
huge=200
n <- ncol(t(data))

#Vectors to save the computational time used by each iteration.
time_iter <- list()       
time_ini <- Sys.time()
comp_time <- c()

rss <- 1e+06
for(i in 1 : (n-3)){
 cat("New iteration:")
 print(i)
 for(f in (i+1):(n-2)){
  for (g in (f+1) : (n-1)){
    for (h in (g+1) : n){ #NOTE: If we would like to compute four archetypes, we would have to add a fourth 
                          #loop and so on and so forth.
      vect_arch_ini=c(i,f,g,h)
      
      k <- length(vect_arch_ini)
      
      x_gvv <- rbind(t(data), rep(huge, n))
      
      zs <- x_gvv[, vect_arch_ini] 
      zs <- as.matrix(zs)
      alphas <- matrix(0, nrow = k, ncol = n)
      for (j in 1 : n){
        alphas[, j] = coef(nnls(zs, x_gvv[,j]))
      }
      
      resid <- zs %*% alphas - x_gvv
      
      auxrss=max(svd(resid)$d) / n
      if(auxrss < rss){
        rss <- auxrss
        mejor=vect_arch_ini
      }
      
    }
  }
  }
  
  time_iter[[i]] <- Sys.time()
  
  if(i == 1){
    comp_time[1] <- difftime(time_iter[[i]], time_ini, units = "mins")
    cat("Computational time in this iteration: \n")
    print(time_iter[[i]] - time_ini)
  }else{
    comp_time[i] <- difftime(time_iter[[i]], time_iter[[i - 1]], units = "mins")
    cat("Computational time in this iteration: \n")
    print(time_iter[[i]] - time_iter[[i - 1]])
  } 
  
  print(mejor)
  print(rss)
}



#Exponential


m=50
set.seed(2014)
r1=rexp(m)
r2=rexp(m)

X=cbind(r1,r2)

K=4

set.seed(2014)
lass10 <- stepArchetypesMod(data=X,k=K,verbose=FALSE,nrep=norep) 
ai <- bestModel(lass10[[1]]) #archetypes
 
an3 <-  stepArchetypoids(K,FALSE,X,lass10,FALSE,K-1) #which_alpha archetypoids

plot(X,xlab='',ylab='',cex.axis=1.5,xaxt="n",yaxt="n",xlim=c(0,3.5),ylim=c(0,4)) 
axis(1,at=seq(0,3.5,.5),labels=seq(0,3.5,.5),cex.axis=1.5)
axis(2,at=seq(0,4,1),labels=seq(0,4,1),cex.axis=1.5)
 text(X[an3[[1]],1],X[an3[[1]],2],la=an3[[1]],cex=1.5)



 x=cmdscale(as.matrix(dist(X)),k=2,eig=T)
x=x$points
norep=20
set.seed(2014)
lass10 <- stepArchetypesMod(data=x,k=K,verbose=FALSE,nrep=norep) 
ai <- bestModel(lass10[[1]]) #archetypes
 
an6 <-  stepArchetypoids(K,FALSE,x,lass10,FALSE,K-1) #which_alpha archetypoids



plot(x,xlab='',ylab='',cex.axis=1.5,xaxt="n",yaxt="n",xlim=c(-1.5,3),ylim=c(-2,2)) 
axis(1,at=seq(-1.5,3,1),labels=seq(-1.5,3,1),cex.axis=1.5)
axis(2,at=seq(-2,2,1),labels=seq(-2,2,1),cex.axis=1.5)
 text(x[an6[[1]],1],x[an6[[1]],2],la=an3[[1]],cex=1.5)
 

